@@ -1,6 +1,6 @@
 #! /bin/sh
 # check script for Xlunzip - Test tool for the lzip_decompress linux module
-# Copyright (C) 2016-2021 Antonio Diaz Diaz.
+# Copyright (C) 2016-2024 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
 # to copy, distribute, and modify it.
@@ -82,33 +82,41 @@ printf "\ntesting decompression..."
 
 for i in "${in_lz}" "${in_em}" ; do
 	"${LZIP}" -t "$i" || test_failed $LINENO "$i"
-	"${LZIP}" -d "$i" -o copy || test_failed $LINENO "$i"
-	cmp in copy || test_failed $LINENO "$i"
-	"${LZIP}" -cd "$i" > copy || test_failed $LINENO "$i"
-	cmp in copy || test_failed $LINENO "$i"
-	"${LZIP}" -d "$i" -o - > copy || test_failed $LINENO "$i"
-	cmp in copy || test_failed $LINENO "$i"
-	"${LZIP}" -d < "$i" > copy || test_failed $LINENO "$i"
-	cmp in copy || test_failed $LINENO "$i"
-	rm -f copy || framework_failure
+	"${LZIP}" -d "$i" -o out || test_failed $LINENO "$i"
+	cmp in out || test_failed $LINENO "$i"
+	"${LZIP}" -cd "$i" > out || test_failed $LINENO "$i"
+	cmp in out || test_failed $LINENO "$i"
+	"${LZIP}" -d "$i" -o - > out || test_failed $LINENO "$i"
+	cmp in out || test_failed $LINENO "$i"
+	"${LZIP}" -d < "$i" > out || test_failed $LINENO "$i"
+	cmp in out || test_failed $LINENO "$i"
+	rm -f out || framework_failure
 done
 
+lines=`"${LZIP}" -tvv "${in_em}" 2>&1 | wc -l` || test_failed $LINENO
+[ "${lines}" -eq 1 ] || test_failed $LINENO "${lines}"
+
+cat "${in_lz}" > out.lz || framework_failure
+"${LZIP}" -dk out.lz || test_failed $LINENO
+cmp in out || test_failed $LINENO
+rm -f out || framework_failure
+"${LZIP}" -cd "${fox_lz}" > fox || test_failed $LINENO
+cat fox > copy || framework_failure
 cat "${in_lz}" > copy.lz || framework_failure
-"${LZIP}" -dk copy.lz || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-printf "to be overwritten" > copy || framework_failure
-"${LZIP}" -d copy.lz 2> /dev/null
+"${LZIP}" -d copy.lz out.lz 2> /dev/null	# skip copy, decompress out
 [ $? = 1 ] || test_failed $LINENO
+[ ! -e out.lz ] || test_failed $LINENO
+cmp fox copy || test_failed $LINENO
+cmp in out || test_failed $LINENO
 "${LZIP}" -df copy.lz || test_failed $LINENO
 [ ! -e copy.lz ] || test_failed $LINENO
 cmp in copy || test_failed $LINENO
+rm -f copy out || framework_failure
 
-printf "to be overwritten" > copy || framework_failure
-"${LZIP}" -d -o copy < "${in_lz}" 2> /dev/null
-[ $? = 1 ] || test_failed $LINENO
-"${LZIP}" -df -o copy < "${in_lz}" || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-rm -f out copy || framework_failure
+printf "to be overwritten" > out || framework_failure
+"${LZIP}" -df -o out < "${in_lz}" || test_failed $LINENO
+cmp in out || test_failed $LINENO
+rm -f out || framework_failure
 "${LZIP}" -d -o ./- "${in_lz}" || test_failed $LINENO
 cmp in ./- || test_failed $LINENO
 rm -f ./- || framework_failure
@@ -117,70 +125,71 @@ cmp in ./- || test_failed $LINENO
 rm -f ./- || framework_failure
 
 cat "${in_lz}" > anyothername || framework_failure
-"${LZIP}" -dv - anyothername - < "${in_lz}" > copy 2> /dev/null ||
+"${LZIP}" -dv - anyothername - < "${in_lz}" > out 2> /dev/null ||
 	test_failed $LINENO
-cmp in copy || test_failed $LINENO
+cmp in out || test_failed $LINENO
 cmp in anyothername.out || test_failed $LINENO
-rm -f copy anyothername.out || framework_failure
+rm -f out anyothername.out || framework_failure
 
 "${LZIP}" -tq in "${in_lz}"
 [ $? = 2 ] || test_failed $LINENO
 "${LZIP}" -tq nx_file.lz "${in_lz}"
 [ $? = 1 ] || test_failed $LINENO
-"${LZIP}" -cdq in "${in_lz}" > copy
+"${LZIP}" -cdq in "${in_lz}" > out
 [ $? = 2 ] || test_failed $LINENO
-cat copy in | cmp in - || test_failed $LINENO
-"${LZIP}" -cdq nx_file.lz "${in_lz}" > copy
+cat out in | cmp in - || test_failed $LINENO		# out must be empty
+"${LZIP}" -cdq nx_file.lz "${in_lz}" > out	# skip nx_file, decompress in
 [ $? = 1 ] || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-rm -f copy || framework_failure
-cat "${in_lz}" > copy.lz || framework_failure
-"${LZIP}" -dq in copy.lz
+cmp in out || test_failed $LINENO
+rm -f out || framework_failure
+cat "${in_lz}" > out.lz || framework_failure
+"${LZIP}" -dq in out.lz
 [ $? = 2 ] || test_failed $LINENO
-[ -e copy.lz ] || test_failed $LINENO
-[ ! -e copy ] || test_failed $LINENO
+[ -e out.lz ] || test_failed $LINENO
+[ ! -e out ] || test_failed $LINENO
 [ ! -e in.out ] || test_failed $LINENO
-"${LZIP}" -dq nx_file.lz copy.lz
+"${LZIP}" -dq nx_file.lz out.lz
 [ $? = 1 ] || test_failed $LINENO
-[ ! -e copy.lz ] || test_failed $LINENO
+[ ! -e out.lz ] || test_failed $LINENO
 [ ! -e nx_file ] || test_failed $LINENO
-cmp in copy || test_failed $LINENO
+cmp in out || test_failed $LINENO
+rm -f out || framework_failure
 
 cat in in > in2 || framework_failure
 "${LZIP}" -t "${in_lz}" "${in_lz}" || test_failed $LINENO
-"${LZIP}" -cd "${in_lz}" "${in_lz}" -o out > copy2 || test_failed $LINENO
+"${LZIP}" -cd "${in_lz}" "${in_lz}" -o out > out2 || test_failed $LINENO
 [ ! -e out ] || test_failed $LINENO			# override -o
-cmp in2 copy2 || test_failed $LINENO
-rm -f copy2 || framework_failure
-"${LZIP}" -d "${in_lz}" "${in_lz}" -o copy2 || test_failed $LINENO
-cmp in2 copy2 || test_failed $LINENO
-rm -f copy2 || framework_failure
+cmp in2 out2 || test_failed $LINENO
+rm -f out2 || framework_failure
+"${LZIP}" -d "${in_lz}" "${in_lz}" -o out2 || test_failed $LINENO
+cmp in2 out2 || test_failed $LINENO
+rm -f out2 || framework_failure
 
-cat "${in_lz}" "${in_lz}" > copy2.lz || framework_failure
-printf "\ngarbage" >> copy2.lz || framework_failure
-"${LZIP}" -tvvvv copy2.lz 2> /dev/null || test_failed $LINENO
-printf "to be overwritten" > copy2 || framework_failure
-"${LZIP}" -df copy2.lz || test_failed $LINENO
-cmp in2 copy2 || test_failed $LINENO
+cat "${in_lz}" "${in_lz}" > out2.lz || framework_failure
+printf "\ngarbage" >> out2.lz || framework_failure
+"${LZIP}" -tvvvv out2.lz 2> /dev/null || test_failed $LINENO
+printf "to be overwritten" > out2 || framework_failure
+"${LZIP}" -df out2.lz || test_failed $LINENO
+cmp in2 out2 || test_failed $LINENO
 
 printf "\ntesting bad input..."
 
 headers='LZIp LZiP LZip LzIP LzIp LziP lZIP lZIp lZiP lzIP'
 body='\001\014\000\203\377\373\377\377\300\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000$\000\000\000\000\000\000\000'
-cat "${in_lz}" > int.lz
-printf "LZIP${body}" >> int.lz
+cat "${in_lz}" > int.lz || framework_failure
+printf "LZIP${body}" >> int.lz || framework_failure
 if "${LZIP}" -tq int.lz ; then
 	for header in ${headers} ; do
-		printf "${header}${body}" > int.lz	# first member
-		"${LZIP}" -tq int.lz
+		printf "${header}${body}" > int.lz || framework_failure
+		"${LZIP}" -tq int.lz			# first member
 		[ $? = 2 ] || test_failed $LINENO ${header}
 		"${LZIP}" -tq < int.lz
 		[ $? = 2 ] || test_failed $LINENO ${header}
 		"${LZIP}" -cdq int.lz > /dev/null
 		[ $? = 2 ] || test_failed $LINENO ${header}
-		cat "${in_lz}" > int.lz
-		printf "${header}${body}" >> int.lz	# trailing data
-		"${LZIP}" -tq int.lz
+		cat "${in_lz}" > int.lz || framework_failure
+		printf "${header}${body}" >> int.lz || framework_failure
+		"${LZIP}" -tq int.lz			# trailing data
 		[ $? = 2 ] || test_failed $LINENO ${header}
 		"${LZIP}" -tq < int.lz
 		[ $? = 2 ] || test_failed $LINENO ${header}
@@ -198,7 +207,6 @@ for i in fox_v2.lz fox_s11.lz fox_de20.lz \
 	[ $? = 2 ] || test_failed $LINENO $i
 done
 
-"${LZIP}" -cd "${fox_lz}" > fox || test_failed $LINENO
 for i in fox_bcrc.lz fox_crc0.lz fox_das46.lz fox_mes81.lz ; do
 	"${LZIP}" -cdq "${testdir}"/$i > out
 	[ $? = 2 ] || test_failed $LINENO $i
@@ -216,153 +224,153 @@ if dd if=in3.lz of=trunc.lz bs=14752 count=1 2> /dev/null &&
 		[ $? = 2 ] || test_failed $LINENO $i
 		"${LZIP}" -tq < trunc.lz
 		[ $? = 2 ] || test_failed $LINENO $i
-		"${LZIP}" -cdq trunc.lz > out
+		"${LZIP}" -cdq trunc.lz > /dev/null
 		[ $? = 2 ] || test_failed $LINENO $i
-		"${LZIP}" -dq < trunc.lz > out
+		"${LZIP}" -dq < trunc.lz > /dev/null
 		[ $? = 2 ] || test_failed $LINENO $i
 	done
 else
 	printf "\nwarning: skipping truncation test: 'dd' does not work on your system."
 fi
-rm -f trunc.lz out || framework_failure
+rm -f trunc.lz || framework_failure
 
 cat "${in_lz}" > ingin.lz || framework_failure
 printf "g" >> ingin.lz || framework_failure
 cat "${in_lz}" >> ingin.lz || framework_failure
 "${LZIP}" -t ingin.lz || test_failed $LINENO
 "${LZIP}" -t < ingin.lz || test_failed $LINENO
-"${LZIP}" -cd ingin.lz > copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-"${LZIP}" -d < ingin.lz > copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
+"${LZIP}" -cd ingin.lz > out || test_failed $LINENO
+cmp in out || test_failed $LINENO
+"${LZIP}" -d < ingin.lz > out || test_failed $LINENO
+cmp in out || test_failed $LINENO
 
 printf "\ntesting linux options..."
 
 for i in "" =1 =2 =4 =16 =64 =256 =1024 =4096 =16384 =65536 ; do
 	"${LZIP}" -t --insize$i "${in_lz}" || test_failed $LINENO $i
-	"${LZIP}" -cd --insize$i "${in_lz}" > copy || test_failed $LINENO $i
-	cmp in copy || test_failed $LINENO $i
+	"${LZIP}" -cd --insize$i "${in_lz}" > out || test_failed $LINENO $i
+	cmp in out || test_failed $LINENO $i
 done
 
 for i in =1 =2 =4 =16 =64 =256 =1024 =4096 ; do
 	"${LZIP}" -tq --insize$i --nofill "${in_lz}"
 	[ $? = 2 ] || test_failed $LINENO $i
-	"${LZIP}" -cdq --insize$i --nofill "${in_lz}" > copy
+	"${LZIP}" -cdq --insize$i --nofill "${in_lz}" > out
 	[ $? = 2 ] || test_failed $LINENO $i
-	cmp -s in copy && test_failed $LINENO $i
+	cmp -s in out && test_failed $LINENO $i
 done
 
 for i in "" =16384 =65536 ; do
 	"${LZIP}" -t --insize$i --nofill "${in_lz}" || test_failed $LINENO $i
-	"${LZIP}" -cd --insize$i --nofill "${in_lz}" > copy ||
+	"${LZIP}" -cd --insize$i --nofill "${in_lz}" > out ||
 		test_failed $LINENO $i
-	cmp in copy || test_failed $LINENO $i
+	cmp in out || test_failed $LINENO $i
 done
 
 for i in =18KiB =65536 =262144 ; do
 	"${LZIP}" -t --outsize$i "${in_lz}" || test_failed $LINENO $i
-	"${LZIP}" -cd --outsize$i "${in_lz}" > copy ||
+	"${LZIP}" -cd --outsize$i "${in_lz}" > out ||
 		test_failed $LINENO $i
-	cmp in copy || test_failed $LINENO $i
+	cmp in out || test_failed $LINENO $i
 done
 
 for i in =36388 =65536 =262144 ; do
 	"${LZIP}" -t --outsize$i --noflush "${in_lz}" ||
 		test_failed $LINENO $i
-	"${LZIP}" -cd --outsize$i --noflush "${in_lz}" > copy ||
+	"${LZIP}" -cd --outsize$i --noflush "${in_lz}" > out ||
 		test_failed $LINENO $i
-	cmp in copy || test_failed $LINENO $i
+	cmp in out || test_failed $LINENO $i
 done
 
 for i in =36387 ; do
 	"${LZIP}" -tq --outsize$i --noflush "${in_lz}"
 	[ $? = 1 ] || test_failed $LINENO $i
-	"${LZIP}" -cdq --outsize$i --noflush "${in_lz}" > copy
+	"${LZIP}" -cdq --outsize$i --noflush "${in_lz}" > out
 	[ $? = 1 ] || test_failed $LINENO $i
-	cmp -s in copy && test_failed $LINENO $i
+	cmp -s in out && test_failed $LINENO $i
 done
 
 for i in =4096 =16384 ; do
 	"${LZIP}" -tq --outsize$i --noflush "${in_lz}"
 	[ $? = 2 ] || test_failed $LINENO $i
-	"${LZIP}" -cdq --outsize$i --noflush "${in_lz}" > copy
+	"${LZIP}" -cdq --outsize$i --noflush "${in_lz}" > out
 	[ $? = 2 ] || test_failed $LINENO $i
-	cmp -s in copy && test_failed $LINENO $i
+	cmp -s in out && test_failed $LINENO $i
 done
 
 "${LZIP}" -t --insize --outsize=36388 --nofill --noflush "${in_lz}" ||
 	test_failed $LINENO
-"${LZIP}" -cd --insize --outsize=36388 --nofill --noflush "${in_lz}" > copy ||
+"${LZIP}" -cd --insize --outsize=36388 --nofill --noflush "${in_lz}" > out ||
 	test_failed $LINENO
-cmp in copy || test_failed $LINENO
+cmp in out || test_failed $LINENO
 
 "${LZIP}" -tq --nofill "${in_lz}"
 [ $? = 2 ] || test_failed $LINENO
-"${LZIP}" -cdq --nofill "${in_lz}" > copy
+"${LZIP}" -cdq --nofill "${in_lz}" > out
 [ $? = 2 ] || test_failed $LINENO
-cmp -s in copy && test_failed $LINENO
+cmp -s in out && test_failed $LINENO
 "${LZIP}" -tq --noflush "${in_lz}"
 [ $? = 1 ] || test_failed $LINENO
-"${LZIP}" -cdq --noflush "${in_lz}" > copy
+"${LZIP}" -cdq --noflush "${in_lz}" > out
 [ $? = 1 ] || test_failed $LINENO
-cmp -s in copy && test_failed $LINENO
+cmp -s in out && test_failed $LINENO
 
 # decompress in place
-rm -f copy copy2 || framework_failure
+rm -f out out2 || framework_failure
 "${LZIP}" -t --in-place "${in_lz}" || test_failed $LINENO
 "${LZIP}" -t --in-place < "${in_lz}" || test_failed $LINENO
-"${LZIP}" -d --in-place "${in_lz}" -o copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-rm -f copy || framework_failure
-"${LZIP}" -d --in-place < "${in_lz}" -o copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-"${LZIP}" -cd --in-place "${in_lz}" > copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-"${LZIP}" -cd --in-place < "${in_lz}" > copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
+"${LZIP}" -d --in-place "${in_lz}" -o out || test_failed $LINENO
+cmp in out || test_failed $LINENO
+rm -f out || framework_failure
+"${LZIP}" -d --in-place < "${in_lz}" -o out || test_failed $LINENO
+cmp in out || test_failed $LINENO
+"${LZIP}" -cd --in-place "${in_lz}" > out || test_failed $LINENO
+cmp in out || test_failed $LINENO
+"${LZIP}" -cd --in-place < "${in_lz}" > out || test_failed $LINENO
+cmp in out || test_failed $LINENO
 "${LZIP}" -t --in-place "${in_lz}" "${in_lz}" || test_failed $LINENO
-"${LZIP}" -d --in-place "${in_lz}" "${in_lz}" -o copy2 || test_failed $LINENO
-cmp in2 copy2 || test_failed $LINENO
-"${LZIP}" -cd --in-place "${in_lz}" "${in_lz}" > copy2 || test_failed $LINENO
-cmp in2 copy2 || test_failed $LINENO
+"${LZIP}" -d --in-place "${in_lz}" "${in_lz}" -o out2 || test_failed $LINENO
+cmp in2 out2 || test_failed $LINENO
+"${LZIP}" -cd --in-place "${in_lz}" "${in_lz}" > out2 || test_failed $LINENO
+cmp in2 out2 || test_failed $LINENO
 
 # decompress multimember in place
 cat in in in > in3 || framework_failure
 for i in in2 in3 ; do
 	"${LZIP}" -t --in-place $i.lz || test_failed $LINENO
 	"${LZIP}" -t --in-place < $i.lz || test_failed $LINENO
-	rm -f copy || framework_failure
-	"${LZIP}" -d --in-place $i.lz -o copy || test_failed $LINENO
-	cmp $i copy || test_failed $LINENO
-	rm -f copy || framework_failure
-	"${LZIP}" -d --in-place < $i.lz -o copy || test_failed $LINENO
-	cmp $i copy || test_failed $LINENO
-	"${LZIP}" -cd --in-place $i.lz > copy || test_failed $LINENO
-	cmp $i copy || test_failed $LINENO
-	"${LZIP}" -cd --in-place < $i.lz > copy || test_failed $LINENO
-	cmp $i copy || test_failed $LINENO
+	rm -f out || framework_failure
+	"${LZIP}" -d --in-place $i.lz -o out || test_failed $LINENO
+	cmp $i out || test_failed $LINENO
+	rm -f out || framework_failure
+	"${LZIP}" -d --in-place < $i.lz -o out || test_failed $LINENO
+	cmp $i out || test_failed $LINENO
+	"${LZIP}" -cd --in-place $i.lz > out || test_failed $LINENO
+	cmp $i out || test_failed $LINENO
+	"${LZIP}" -cd --in-place < $i.lz > out || test_failed $LINENO
+	cmp $i out || test_failed $LINENO
 done
 "${LZIP}" -tq --in-place ingin.lz
 [ $? = 2 ] || test_failed $LINENO
-rm -f copy copy2 ingin.lz in2 in2.lz in3 in3.lz || framework_failure
+rm -f out out2 ingin.lz in2 in2.lz in3 in3.lz || framework_failure
 
 cat "${in_lz}" > inz.lz || framework_failure
 counter=0
 while [ ${counter} -lt 20 ] ; do
 	cat "${zero_lz}" >> inz.lz || framework_failure
 	"${LZIP}" -t --in-place inz.lz || test_failed $LINENO
-	"${LZIP}" -cd --in-place inz.lz > copy || test_failed $LINENO
-	cmp in copy || test_failed $LINENO
+	"${LZIP}" -cd --in-place inz.lz > out || test_failed $LINENO
+	cmp in out || test_failed $LINENO
 	counter=$((counter+1))
 done
-rm -f copy inz.lz || framework_failure
+rm -f out inz.lz || framework_failure
 
 # decompress with trailing data in place
 cat "${in_lz}" in in in in > int.lz || framework_failure
 "${LZIP}" -t --in-place int.lz || test_failed $LINENO
-"${LZIP}" -cd --in-place int.lz > copy || test_failed $LINENO
-cmp in copy || test_failed $LINENO
-rm -f copy int.lz || framework_failure
+"${LZIP}" -cd --in-place int.lz > out || test_failed $LINENO
+cmp in out || test_failed $LINENO
+rm -f out int.lz || framework_failure
 
 echo
 if [ ${fail} = 0 ] ; then
